#========================================================================================
# package      DE_EPAGES::Packlink::t::GrantPacklinkAuthorization_si
# base         DE_EPAGES::Shop::API::TestImportXML
# state        public
#----------------------------------------------------------------------------------------
# description  System Integration Test for GrantPacklinkAuthorization
#========================================================================================
package DE_EPAGES::Packlink::t::GrantPacklinkAuthorization_si;
use base DE_EPAGES::Shop::API::TestImportXML;

use Readonly;
use URI;
use URI::QueryParam;
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Packlink::API::Connector;
use DE_EPAGES::Packlink::API::User qw ( CreatePacklinkUser GetPacklinkAuthToken );
use Test::MockObject::Extends;
use Test::More;
use DE_EPAGES::Packlink::UI::Shop;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Object::API::Language qw (GetPKeyLanguageByCode);
use strict;

use constant SHIPPING_METHOD_NAME_IN_SHOP => 'Change this name';
use constant PACKLINK_ID => '3453564646454';
use constant SHIPPING_TYPE_FIX_PRICE => 'ShippingMethodFixPrice';

# test plan
sub test {
    my $self = shift;

    local $DE_EPAGES::Object::API::Language::LANGUAGEID = GetPKeyLanguageByCode('en');

    $self->test_PacklinkCredentials();

    done_testing();
}

sub test_PacklinkCredentials {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Password = 'password';
    my $ShopPath = $self->shopPath();
    my $Shop = LoadObjectByPath($ShopPath);

    my $Connector = $self->_createConnector($Shop);
    my $User = $self->_createPacklinkUser($Shop, $Password);
    my $AuthToken = $self->_getPacklinkAuthToken($Shop,$Connector,$User, $Password);
    my $ShippingFulfillmentType = $self->_getShippingFulfillmentType($Shop);

    like($User->pathString, '/PacklinkConnector/' ,"$SubTestName - Check that a user called PacklinkConnector is created");
    like($Connector->GetPacklinkSoapURL($Shop), '/soap$/' ,"$SubTestName - Check Shipo Soap URL sent"); ## no critic qw(RequireInterpolationOfMetachars)
    like($AuthToken, '/^[A-Z0-9]+$/i', "$SubTestName - Check that Authtoken contains data"); ## no critic qw(RequireInterpolationOfMetachars)
    is($ShippingFulfillmentType->get('AuthorizationURL'), 'https://pro.packlink.de/landing/epages/connect.html',
       "$SubTestName - We are getting the URL of Packlink where the credentials have to be sent");
}

sub shopAlias {
    return 'TestShopPacklink';
}

sub importXMLPath {
    return 'ShopXMLs/ShopWithPacklink.xml';
}

#========================================================================================
sub _createPacklinkUser {
    my $self = shift;
    my ($Shop,$Password) = @_;
    my $User = CreatePacklinkUser($Shop, $Password);
    return $User;
}

#========================================================================================
sub _getPacklinkAuthToken {
    my $self = shift;
    my ($Shop,$Connector,$User,$Password) = @_;
    my $AuthToken = GetPacklinkAuthToken($User->pathString, $Password, $Connector->GetPacklinkSoapURL($Shop));
    return $AuthToken;
}

#========================================================================================
sub _getShippingFulfillmentType {
    my $self = shift;
    my ($Shop) = @_;

    my $PacklinkProviderMethod = $self->_getPacklinkShippingProviderMethod($Shop);
    my $ShippingFulfillmentType = $PacklinkProviderMethod->get('ShippingFulfillmentType');

    return $ShippingFulfillmentType;
}

#========================================================================================
sub _createConnector {
    my $self = shift;
    my ($Shop) = @_;

    my $PacklinkShippingProviderMethod = $self->_getPacklinkShippingProviderMethod($Shop);
    return DE_EPAGES::Packlink::API::Connector->new($PacklinkShippingProviderMethod);
}

#========================================================================================
sub _getPacklinkShippingProviderMethod {
    my $self = shift;
    my ($Shop) = @_;

    my $ProviderMethodsFolder = $Shop->folder('ShippingProviderMethods');
    my $PacklinkProviderMethod = $ProviderMethodsFolder->existsChild('Packlink') ? $ProviderMethodsFolder->child('Packlink') : undef;

    return $PacklinkProviderMethod;
}

__PACKAGE__->runScript();

1;
