#========================================================================================
# package      DE_EPAGES::Packlink::t::unit::Connector
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::Packlink::API::Connector
#========================================================================================
package DE_EPAGES::Packlink::t::unit::Connector;
use base DE_EPAGES::Core::API::Test;
use Test::More;
use Test::Exception;
use DE_EPAGES::Packlink::API::Connector;
use DE_EPAGES::Test::Mock::EpagesObject;
use strict;

sub test {
    my $self = shift;
    $self->test_Positive_getMethod();

    $self->test_Positive_isConnectionSetupDone();
    $self->test_Negative_isConnectionSetupDone_notDone();

    $self->test_Positive_deleteConnection();

    $self->test_Positive_SendPacklinkAuthCredentials();
    $self->test_Negative_SendPacklinkAuthCredentials();

    done_testing();
}
## no critic (ProhibitLongLines)
sub test_Positive_getMethod {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ShippingProviderMethodPacklink = $self->ShippingProviderMethodPacklinkWithPacklinkID();
    my $Connector = DE_EPAGES::Packlink::API::Connector->new($ShippingProviderMethodPacklink);

    is( $Connector->getMethod()->id, '12345', "$SubTestName - tests if getMethod returns the ShippingProviderMethod already defined" );
}

sub test_Positive_isConnectionSetupDone {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ShippingProviderMethodPacklink = $self->ShippingProviderMethodPacklinkWithPacklinkID();
    my $Connector = DE_EPAGES::Packlink::API::Connector->new($ShippingProviderMethodPacklink);

    is($Connector->isConnectionSetupDone(), '1', "$SubTestName - tests if the connection is already done" );
}

sub test_Negative_isConnectionSetupDone_notDone {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ShippingProviderMethodPacklink = $self->ShippingProviderMethodPacklinkWithoutPacklinkID();
    my $Connector = DE_EPAGES::Packlink::API::Connector->new($ShippingProviderMethodPacklink);

    is($Connector->isConnectionSetupDone(), '0', "$SubTestName - tests if the connection is already done" );
}

sub test_Positive_deleteConnection {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ShippingProviderMethodPacklink = $self->ShippingProviderMethodPacklinkWithPacklinkID();
    my $Connector = DE_EPAGES::Packlink::API::Connector->new($ShippingProviderMethodPacklink);

    my $ConnectorMocked = Test::MockObject::Extends->new($Connector);

    $ConnectorMocked->mock('_deleteShopPacklinkUser', sub {
        shift;
        return 1;
    });
    $ConnectorMocked->mock('isUsable', sub {
        shift;
        return 1;
    });

    lives_ok(sub{ $ConnectorMocked->deleteConnection(),},"$SubTestName - test delete connection method");
}

sub test_Positive_SendPacklinkAuthCredentials {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ShippingProviderMethodPacklink = $self->ShippingProviderMethodPacklinkWithPacklinkID();
    my $ShippingFulfillmentType = $self->getShippingFulfillmentType();
    my $Connector = DE_EPAGES::Packlink::API::Connector->new($ShippingProviderMethodPacklink);
    my $ConnectorMocked = Test::MockObject::Extends->new($Connector);

    $ConnectorMocked->mock('_createUserAgent', sub {
        return _createOKLWPUserAgent();
    });

    $ConnectorMocked->mock('_logConnectorData', sub {
        return 1;
    });

    my $IsInternalTestShop = 1;
    my $hAuthCredentials   = $self->_createHAuthCredentials($IsInternalTestShop, $ShippingFulfillmentType);

    lives_ok(sub{ $ConnectorMocked->SendPacklinkAuthCredentials($hAuthCredentials);},
            "$SubTestName - test the sending of the Packlink Credentials");
}

sub test_Negative_SendPacklinkAuthCredentials {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ShippingProviderMethodPacklink = $self->ShippingProviderMethodPacklinkWithPacklinkID();
    my $ShippingFulfillmentType = $self->getShippingFulfillmentType();
    my $Connector = DE_EPAGES::Packlink::API::Connector->new($ShippingProviderMethodPacklink);
    my $ConnectorMocked = Test::MockObject::Extends->new($Connector);

    $ConnectorMocked->mock('_createUserAgent', sub {
        return _createErrorLWPUserAgent();
    });

    $ConnectorMocked->mock('_logConnectorData', sub {
        return 1;
    });

    my $IsInternalTestShop = 1;
    my $hAuthCredentials   = $self->_createHAuthCredentials($IsInternalTestShop, $ShippingFulfillmentType);

    throws_ok(sub{ $ConnectorMocked->SendPacklinkAuthCredentials($hAuthCredentials);}
    , qr/POST_NO_OK/ ,"$SubTestName - test the sending of the Packlink Credentials with Error in the POST");
}

#----------------------------------------------------------------------------------------
sub ShippingProviderMethodPacklinkWithPacklinkID {
    my $self = shift;

    my $Attributes = {
        'ID'                    => 12345,
        'PacklinkID'              => 98765,
    };

    my $Result =
         DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Packlink::API::Object::ShippingProviderMethodPacklink', $Attributes);

    $Result->mock('getSite', sub {
        shift;
        return 1;
    });

    $Result->mock('isUsable', sub {
        shift;
        return 1;
    });

    return $Result;
}

#----------------------------------------------------------------------------------------
sub ShippingProviderMethodPacklinkWithoutPacklinkID {
    my $self = shift;

    my $Attributes = {
        'ID'                    => 12345,
        'PacklinkID'              => '',
    };

    my $Result =
         DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Packlink::API::Object::ShippingProviderMethodPacklink', $Attributes);

    $Result->mock('getSite', sub {
        shift;
        return 1;
    });

    $Result->mock('isUsable', sub {
        shift;
        return 0;
    });

    return $Result;
}

#----------------------------------------------------------------------------------------
sub _createOKLWPUserAgent {
    my $LWPUserAgent = LWP::UserAgent->new();
    my $MockedUserAgent = Test::MockObject::Extends->new( $LWPUserAgent );
    $MockedUserAgent->mock('request',sub{
                my $self = shift;
                my ($HTTPRequest) = @_;
                $MockedUserAgent->{'HTTPRequest'} = $HTTPRequest;
                return $self;
        });
    $MockedUserAgent->mock('content',sub{
                return " \n";
        });
    $MockedUserAgent->mock('is_success',sub{
                return 1;
        });
    return $MockedUserAgent;
}
#----------------------------------------------------------------------------------------
sub _createErrorLWPUserAgent {
    my $LWPUserAgent = LWP::UserAgent->new();
    my $MockedUserAgent = Test::MockObject::Extends->new( $LWPUserAgent );
    $MockedUserAgent->mock('request',sub{
                my $self = shift;
                my ($HTTPRequest) = @_;
                $MockedUserAgent->{'HTTPRequest'} = $HTTPRequest;
                return $self;
        });
    $MockedUserAgent->mock('content',sub{
                return " \n";
        });
    $MockedUserAgent->mock('is_success',sub{
                return 0;
        });
    return $MockedUserAgent;
}

#----------------------------------------------------------------------------------------
sub getShippingFulfillmentType {
    my $self = shift;
    my $Attributes = {
        'AuthorizationURL'      => 'www.authurl.cat',
        'TestAuthorizationURL'  => 'www.test.cat',
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentType', $Attributes)
}

#----------------------------------------------------------------------------------------
sub _createHAuthCredentials {
    my $self = shift;
    my ( $IsInternalTestShop, $ShippingFulfillmentType ) = @_;

    my $hAuthCredentials = {
            'IsInternalTestShop'        => $IsInternalTestShop,
            'ShippingFulfillmentType'   => $ShippingFulfillmentType,
            'AuthToken'                 => 'AuthTokenTest',
            'PacklinkID'                => '12345',
            'ProviderKey'               => 'ProviderTest'
    };

    return $hAuthCredentials;
}


## use critic (ProhibitLongLines)
__PACKAGE__->runScript();